import sys, os
import gi
from .utils import service_status, service_enable, _

gi.require_version('Gtk', '4.0')
gi.require_version('Adw', '1')

from gi.repository import Gtk, Gio, Adw
from .window import XimperUnifiedThemeSwitcherGuiGtk4Window

class ServiceNeededDialog(Adw.AlertDialog):
    def __init__(self, parent_window):
        super().__init__()
        self.parent_window = parent_window
        
        self.set_heading(_("The user service is disabled.")),
        self.set_body(_("Do you want to enable it?"))
        
        self.add_response("yes", _("Yes"))
        self.add_response("no", _("No"))

        self.connect("response", self.on_response)

    def present(self):
        """Present the dialog to the user."""
        super().present(self.parent_window)
        
    def on_response(self, dialog, response):
        print(f"response: {response}")
        
        if response == "yes":
            service_enable("ximper-unified-theme-switcher.service")
            dialog.close()
            
        elif response in ("no", "close"):
            dialog.close()
            self.parent_window.close()


class XimperUnifiedThemeSwitcherGuiGtk4Application(Adw.Application):
    """The main application singleton class."""

    def __init__(self):
        super().__init__(application_id='ru.ximperlinux.UnifiedThemeSwitcher',
                         flags=Gio.ApplicationFlags.DEFAULT_FLAGS)
        self.create_action('quit', lambda *_: self.quit(), ['<primary>q'])
        self.create_action('about', self.on_about_action)
        self.create_action('preferences', self.on_preferences_action)

    def do_activate(self):
        """Called when the application is activated.

        We raise the application's main window, creating it if
        necessary.
        """
        win = self.props.active_window
        
        if not win:
            win = XimperUnifiedThemeSwitcherGuiGtk4Window(application=self)
        win.present()
        
        if service_status("ximper-unified-theme-switcher.service") == False:
            print("service not found")
            dialog = ServiceNeededDialog(win)
            dialog.present()            

    def on_about_action(self, widget, _):
        """Callback for the app.about action."""
        about = Adw.AboutWindow(transient_for=self.props.active_window,
                                application_name='Ximper\nUnified Theme Switcher\nGUI',
                                application_icon='ru.ximperlinux.UnifiedThemeSwitcher',
                                developer_name='Ximper Linux',
                                version='0.1.0',
                                developers=['Ximper', 'Fiersik'],
                                issue_url="https://gitlab.eterfund.ru/ximperlinux/ximper-unified-theme-switcher/issues",
                                website="https://gitlab.eterfund.ru/ximperlinux/ximper-unified-theme-switcher",
                                copyright='© 2024 Etersoft')
        about.present()

    def on_preferences_action(self, widget, _):
        """Callback for the app.preferences action."""
        print('app.preferences action activated')

    def create_action(self, name, callback, shortcuts=None):
        """Add an application action.

        Args:
            name: the name of the action
            callback: the function to be called when the action is
              activated
            shortcuts: an optional list of accelerators
        """
        action = Gio.SimpleAction.new(name, None)
        action.connect("activate", callback)
        self.add_action(action)
        if shortcuts:
            self.set_accels_for_action(f"app.{name}", shortcuts)


def main(version):
    """The application's entry point."""
    app = XimperUnifiedThemeSwitcherGuiGtk4Application()
    return app.run(sys.argv)
