import sys, os, re
import subprocess
import functools

import gi
gi.require_version('Gtk', '4.0')
gi.require_version('Adw', '1')

from gi.repository import Gtk

import gettext
gettext.textdomain('ximper-unified-theme-switcher-gui')
_ = gettext.gettext

SYSTEM_CONFIG_DIR = "/etc/ximper-unified-theme-switcher"
SYSTEM_CONFIG_FILE = os.path.join(SYSTEM_CONFIG_DIR, "themes")

HOME_CONFIG_DIR = os.path.join(os.path.expanduser("~/.config/ximper-unified-theme-switcher"))
HOME_CONFIG_FILE = os.path.join(HOME_CONFIG_DIR, "themes")

def create_config():
    try:
        if os.geteuid() == 0:
            os.makedirs(SYSTEM_CONFIG_DIR, exist_ok=True)
            open(SYSTEM_CONFIG_FILE, 'a').close()
            return SYSTEM_CONFIG_FILE
        else:
            os.makedirs(HOME_CONFIG_DIR, exist_ok=True)
            open(HOME_CONFIG_FILE, 'a').close()
            return HOME_CONFIG_FILE
    except Exception as e:
        print(f"Error creating config file: {e}")
        return None

def cfg_check():
    try:
        if os.path.isfile(SYSTEM_CONFIG_FILE) and os.access(SYSTEM_CONFIG_FILE, os.W_OK):
            return SYSTEM_CONFIG_FILE
        elif os.path.isfile(HOME_CONFIG_FILE):
            return HOME_CONFIG_FILE
        else:
            return create_config()
    except Exception as e:
        print(f"Error checking config file: {e}")
        return None

def read_config(key):
    with open(cfg_check(), 'r') as file:
        for line in file:
            line = line.strip()
            if line:
                k, v = line.split('=')
                if k.strip() == key:
                    return v.strip()

def write_config(key, value):
    config_file = cfg_check()
    lines = []

    with open(config_file, 'r') as file:
        lines = file.readlines()

    with open(config_file, 'w') as file:
        found = False
        for line in lines:
            if line.strip():
                k, v = line.split('=')
                if k.strip() == key:
                    file.write(f"{key}={value}\n")
                    found = True
                else:
                    file.write(line)
            else:
                file.write(line)
        
        if not found:
            file.write(f"{key}={value}\n")

def make_gtk3_themes_list():
    themes_dirs = [
        "/usr/share/themes/",
        os.path.expanduser("~/.themes/"),
        os.path.expanduser("~/.local/share/themes/")
    ]

    themes_list = []

    for themes_dir in themes_dirs:
        if os.path.exists(themes_dir) and os.listdir(themes_dir):
            themes = os.listdir(themes_dir)
            for theme in themes:
                themes_list.append(theme)
    return themes_list

def make_kv_themes_list():
    themes_dirs = [
        "/usr/share/color-schemes/"
    ]

    themes_list = []

    for themes_dir in themes_dirs:
        if os.path.exists(themes_dir) and os.listdir(themes_dir):
            themes = os.listdir(themes_dir)
            for theme in themes:
                if theme.startswith("Kv"):
                    theme_name = re.sub(r'\.colors$', '', theme)
                    themes_list.append(theme_name)
    return themes_list

def separate_themes(kv_themes_list, gtk3_themes_list):
    kv_light_themes, kv_dark_themes = [], []
    gtk3_light_themes, gtk3_dark_themes = [], []

    def classify_theme(theme, light_themes, dark_themes, themes_list):
        theme_lower = theme.lower()
        if 'light' in theme_lower:
            light_themes.append(theme)
        elif 'dark' in theme_lower:
            dark_themes.append(theme)
        else:
            if (theme + 'light').lower() in map(str.lower, themes_list):
                dark_themes.append(theme)
            elif (theme + 'dark').lower() in map(str.lower, themes_list):
                light_themes.append(theme)
            else:
                light_themes.append(theme)

    for theme in kv_themes_list:
        classify_theme(theme, kv_light_themes, kv_dark_themes, kv_themes_list)

    for theme in gtk3_themes_list:
        classify_theme(theme, gtk3_light_themes, gtk3_dark_themes, gtk3_themes_list)

    return kv_light_themes, kv_dark_themes, gtk3_light_themes, gtk3_dark_themes

def find_string_index(lst, string):
    try:
        index = lst.index(string)
        return index
    except ValueError:
        return 0

def configure_cr_widgets(crs_configurations, selected_func):
    for cr, themes_list, config_key in crs_configurations:
        
        model = Gtk.StringList()
        for theme in themes_list:
            model.append(theme)
        cr.set_model(model)
        
        cr.set_selected(find_string_index(
            themes_list,
            read_config(config_key))
            )
        
        cr.connect(
            'notify::selected-item',
            functools.partial(
                selected_func,
                cfg_key=config_key
                )
            )
        
def service_status(service_name):
    try:
        # Запускаем команду systemctl is-active <service_name>
        result = subprocess.run(['systemctl', '--user', 'is-active', service_name], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        # Проверяем статус
        if result.stdout.decode('utf-8').strip() == 'active':
            return True
        else:
            return False
    except Exception as e:
        print(f"An error occurred: {e}")
        return False

def service_enable(service_name):
    try:
        subprocess.run(['systemctl', '--user', '--now', 'enable', service_name], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    except Exception as e:
        print(f"An error occurred: {e}")

